



class CommitPart extends Entry{

    // Max size of the part excluding the fixed size header
    static final int MAX_NON_HEADER_SIZE = 512;


    // Sequence number of the transaction this commit is for, -1 if not a cloud transaction
    int64_t machineId = -1; // Machine Id of the device that made the commit
    int64_t sequenceNumber = -1; // commit sequence number for this arbitrator
    int64_t transactionSequenceNumber = -1;
    int partNumber = -1; // Parts position in the
    Boolean isLastPart = false;
    char[] data = NULL;

    Pair<Long, Integer> partId = NULL;
    Pair<Long, Long> commitId = NULL;


    CommitPart(Slot s, int64_t _machineId, int64_t _sequenceNumber, int64_t _transactionSequenceNumber, int _partNumber, char[] _data, Boolean _isLastPart) {
        super(s);
        machineId = _machineId;
        sequenceNumber = _sequenceNumber;
        transactionSequenceNumber = _transactionSequenceNumber;
        partNumber = _partNumber;
        isLastPart = _isLastPart;
        data = _data;

        partId = new Pair<Long, Integer>(sequenceNumber, partNumber);
        commitId = new Pair<Long, Long>(machineId, sequenceNumber);
    }

    int getSize() {
        if (data == NULL) {
            return (3 * sizeof(int64_t)) + (2 * sizeof(int32_t)) + (2 * sizeof(char));
        }
        return (3 * sizeof(int64_t)) + (2 * sizeof(int32_t)) + (2 * sizeof(char)) + data.length;
    }

    void setSlot(Slot s) {
        parentslot = s;
    }

    int getPartNumber() {
        return partNumber;
    }

    int getDataSize() {
        return data.length;
    }

    char[] getData() {
        return data;
    }

    Pair<Long, Integer> getPartId() {
        return partId;
    }

    Pair<Long, Long> getCommitId() {
        return commitId;
    }

    Boolean isLastPart() {
        return isLastPart;
    }

    int64_t getMachineId() {
        return machineId;
    }

    int64_t getTransactionSequenceNumber() {
        return transactionSequenceNumber;
    }

    int64_t getSequenceNumber() {
        return sequenceNumber;
    }

    static Entry decode(Slot s, ByteBuffer bb) {
        int64_t machineId = bb.getLong();
        int64_t sequenceNumber = bb.getLong();
        int64_t transactionSequenceNumber = bb.getLong();
        int partNumber = bb.getInt();
        int dataSize = bb.getInt();
        Boolean isLastPart = bb.get() == 1;

        // Get the data
        char[] data = new char[dataSize];
        bb.get(data);

        return new CommitPart(s, machineId, sequenceNumber, transactionSequenceNumber, partNumber, data, isLastPart);
    }

    void encode(ByteBuffer bb) {
        bb.put(Entry.TypeCommitPart);
        bb.putLong(machineId);
        bb.putLong(sequenceNumber);
        bb.putLong(transactionSequenceNumber);
        bb.putInt(partNumber);
        bb.putInt(data.length);

        if (isLastPart) {
            bb.put((char)1);
        } else {
            bb.put((char)0);
        }

        bb.put(data);
    }

    char getType() {
        return Entry.TypeCommitPart;
    }

    Entry getCopy(Slot s) {
        return new CommitPart(s, machineId, sequenceNumber, transactionSequenceNumber, partNumber, data, isLastPart);
    }
}
