
/**
 * KeyValue entry for Slot.
 * @author Brian Demsky <bdemsky@uci.edu>
 * @version 1.0
 */

class KeyValue { /*extends Entry */
	IoTString key;
	IoTString value;

	KeyValue(IoTString _key, IoTString _value) {
		key = _key;
		value = _value;
	}

	IoTString getKey() {
		return key;
	}

	IoTString getValue() {
		return value;
	}

	static KeyValue decode(ByteBuffer bb) {
		int keylength = bb.getInt();
		int valuelength = bb.getInt();
		char[] key = new char[keylength];
		bb.get(key);

		if (valuelength != 0) {
			char[] value = new char[valuelength];
			bb.get(value);
			return new KeyValue(IoTString.shallow(key), IoTString.shallow(value));
		}

		return new KeyValue(IoTString.shallow(key), NULL);
	}

	void encode(ByteBuffer bb) {
		bb.putInt(key.length());

		if (value != NULL) {
			bb.putInt(value.length());
		} else {
			bb.putInt(0);
		}

		bb.put(key.internalBytes());

		if (value != NULL) {
			bb.put(value.internalBytes());
		}
	}

	int getSize() {
		if (value != NULL) {
			return 2 * sizeof(int32_t) + key.length() + value.length();
		}

		return 2 * sizeof(int32_t) + key.length();
	}

	String toString() {
		if (value == NULL) {
			return "NULL";
		}
		return value.toString();
	}

	KeyValue getCopy() {
		return new KeyValue(key, value);
	}
}
